--| MIN_OUT_TEST
--|-----------------------------------------------------------------------------
--   Copyright (C) GunnarGotshalks --  All rights reserved.
--|--| Permission to use, copy, modify, and distribute,without fee this
--| software and its documentation for educational purposes and is
--| hereby granted.
--|-----------------------------------------------------------------------------

note
  description: "Dictionary minimal output test program"
  author: "Gunnar Gotshalks"
  date: "2003 February"
  version: "1.0"

class MIN_OUT_TEST

create make

feature -- Creation

do_precondition_cases : BOOLEAN = false
test_error_messages : BOOLEAN = false

make
  -- Defines the test cases to use
  local
   answer : BOOLEAN
  do
    println("Dictionary Test Started")
  	answer := create_too_small(9, 1)
    create_size(10, 2)
    create_size(11, 3)
    void_precond_cases(10, 4)
    has_precond_cases(10, 5)
    space_precond_cases(10,6)
    one_entry_cases(10,7)
    multiple_entry_cases(10,5,19)
    multiple_entry_cases(10,10,31) -- last entry cases
    println("Test finished")
  end

create_size(size : INTEGER; error_num : INTEGER)
  -- Does DICTIONARY create an object?
  local
   dictionary : DICTIONARY[INTEGER, STRING]
  do
    create dictionary.make(size)
    if  dictionary = void then
      print_error_1(error_num, "Failed to create dictionary of size", size)
    end
    if dictionary.capacity /= size then
      print_error_2( error_num, "Capacity of dictionary is not"
                   , size, "instead it is"
                   , dictionary.capacity)
    end
    if dictionary.count /= 0 then
      print_error_1( error_num, "Count of dictionary is not 0 instead it is"
                   , dictionary.count)
    end
    if not dictionary.is_empty then
      print_error_1(error_num, "Dictionary is not emty, size is"
                   , size)
    end
    if dictionary.is_full then
      print_error_1(error_num, "Dictionary is full, size is"
                   , size)
    end
  end

create_too_small(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Does precondition too small fail?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	if not Result and do_precondition_cases then
  	  create dictionary.make(size)
      print_error_1(error_num, "Create precondition fails on size", size)
    end
  rescue
  	Result := true
    retry
  end

--------------------------------------
feature -- Pre key void tests

void_precond_cases(size : INTEGER; error_num : INTEGER)
-- Test void key trapped as precondition in every relevent routine
  local
    answer : BOOLEAN
  do if do_precondition_cases then
  	answer := has_void_key(size, error_num)
  	answer := at_void_key(size, error_num)
  	answer := infix_at_void_key(size, error_num)
  	answer := put_void_key(size, error_num)
  	answer := put_defensive_void_key(size, error_num)
  	answer := replace_void_key(size, error_num)
  	answer := remove_void_key(size, error_num)
  end
end

has_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : BOOLEAN
  do
  	create dictionary.make(size)
  	if Result = false then
  	  temp_result := dictionary.has(void)
      print_err_msg(error_num, "Has precondition fails on void key")
  	end
  rescue
  	Result := true
    retry
  end

at_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
  	if Result = false then
      create dictionary.make(size)
  	  temp_result := dictionary.at(void)
  	  print_err_msg(error_num, "At precondition fails on void key")
    end
  rescue
    Result := true
    retry
  end

infix_at_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
    if Result = false then
      create dictionary.make(size)
      temp_result := dictionary @ void
      print_err_msg(error_num, "Infix at precondition fails on void key")
    end
  rescue
  	Result := true
    retry
  end

put_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	if Result = false then
      create dictionary.make(size)
  	  dictionary.put(void, "one")
      print_err_msg(error_num, "Put precondition fails on void key")
    end
  rescue
  	Result := true
    retry
  end

put_defensive_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	if Result = false then
      create dictionary.make(size)
      dictionary.put_defensive(void, "one")
      print_err_msg(error_num, "Infix at precondition fails on void key")
    end
  rescue
  	Result := true
    retry
  end

replace_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	if Result = false then
      create dictionary.make(size)
  	  dictionary.replace(void, "one")
      print_err_msg(error_num, "Replace precondition fails on void key")
    end
  rescue
  	Result := true
    retry
  end

remove_void_key(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	if Result = false then
      create dictionary.make(size)
  	  dictionary.remove(void)
      print_err_msg(error_num, "Remove precondition fails on void key")
  	end
  rescue
  	Result := true
    retry
  end

--------------------------------------
feature -- Pre has tests

has_precond_cases(size : INTEGER; error_num : INTEGER)
-- Test has trapped as precondition in every relevent routine
  local
   answer : BOOLEAN
  do if do_precondition_cases then
  	answer := at_not_has(size, error_num)
  	answer := infix_at_not_has(size, error_num)
  	answer := put_has(size, error_num)
  	answer := remove_not_has(size, error_num)
  end
end

at_not_has(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
    if Result = false then
      create dictionary.make(size)
      temp_result := dictionary.at("1")
  	  print_err_msg(error_num, "At precondition fails on not has key")
    end
  rescue
    Result := true
    retry
  end

infix_at_not_has(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
    if Result = false then
      create dictionary.make(size)
      temp_result := dictionary @ "1"
      print_err_msg(error_num, "Infix at precondition fails on not has key")
    end
  rescue
    Result := true
    retry
  end

put_has(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  do
    if Result = false then
      create dictionary.make(size)
      dictionary.put("1","one")
      dictionary.put("1","two")
      print_err_msg(error_num, "Put precondition fails on has key")
    end
  rescue
    Result := true
    retry
  end

remove_not_has(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  do
    if Result = false then
      create dictionary.make(size)
      dictionary.remove("1")
      print_err_msg(error_num, "Remove precondition fails on not has key")
    end
  rescue
    Result := true
    retry
  end

--------------------------------------
feature -- Pre space exists tests

space_precond_cases(size : INTEGER; error_num : INTEGER)
-- Test space_exists trapped as precondition in every relevent routine
  local
   answer : BOOLEAN
  do if do_precondition_cases then
  	answer := put_no_space(size, error_num)
  	answer := put_defensive_no_space(size, error_num)
  end
end

put_no_space(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check space exists precondition.
  local
    dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
  	create dictionary.make(size)
    insert_put(dictionary, size)
    if Result = false then
      dictionary.put(k,1001)
      print_err_msg(error_num, "put precondition fails on no space")
    end
  rescue
    Result := true
    retry
  end

put_defensive_no_space(size : INTEGER; error_num : INTEGER) : BOOLEAN
  -- Check space exists precondition.
  local
    dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
  	create dictionary.make(size)
    insert_put_defensive(dictionary, size)
    if Result = false then
      dictionary.put_defensive(k,1001)
      print_err_msg(error_num, "put defensive precondition fails on no space")
    end
  rescue
    Result := true
    retry
  end

insert_put_defensive( dictionary : DICTIONARY[INTEGER,INTEGER]
                    ; count : INTEGER)
  local k : INTEGER
  do
    from k := 1
  	invariant -- for j : 1..k-1 :: an entry is in the dictionary
  	until k > count
  	loop
  	  dictionary.put_defensive(k,k*2)
  	  k := k + 1
  	variant count + 1 - k
  	end
  end

insert_put( dictionary : DICTIONARY[INTEGER,INTEGER]
          ; count : INTEGER)
  local k : INTEGER
  do
    from k := 1
  	invariant -- for j : 1..k-1 :: an entry is in the dictionary
  	until k > count
  	loop
  	  dictionary.put(k,k*2)
  	  k := k + 1
  	variant count + 1 - k
  	end
  end

--------------------------------------
feature -- One entry tests

one_entry_cases(size : INTEGER; error_num : INTEGER)
  -- Test cases dealing with one entry
  local dictionary : DICTIONARY[STRING, INTEGER]
  do
    create dictionary.make(size)

  -- Check one valid entry inserted correctly
    dictionary.put("1", 1)
    if not dictionary.has("1") or test_error_messages then
      print_err_msg(error_num, "put fails on inserting first value")
    end

  -- Check count is 1 with one entry entered
    if dictionary.count /= 1 or test_error_messages then
      print_error_1(error_num+1, "put count not one on one entry, actual is"
                 , dictionary.count)
    end

  -- Check one valid entry inserted correctly

    if (dictionary.at("1")) /= 1 or test_error_messages then
      print_error_1(error_num+2, "put first entry value not 1, actual at is"
                 , dictionary.at("1"))
    end

    if dictionary @ "1" /= 1 or test_error_messages then
      print_error_1(error_num+3, "put first entry value not 1, actual infix at is"
                 , dictionary @ "1")
    end

  -- Check count is decremented on remove first entry
    dictionary.remove("1")
    if dictionary.count /= 0 or test_error_messages then
      print_error_1(error_num+4, "put first entry remove count is not 0, actual is"
                 , dictionary.count)
    end

  -- Check one valid entry removed correctly
    if dictionary.has("1") or test_error_messages then
      print_err_msg(error_num+5, "put first entry value remove still has string 1")
    end

  -----------------------------
  -- put_defensive cases

    create dictionary.make(size)

  -- Check one valid entry inserted correctly
    dictionary.put_defensive("1", 1)
    if not dictionary.has("1") or test_error_messages then
      print_err_msg(error_num+6, "put defensive fails on inserting first value")
    end

-- Check count is 1 with one entry entered
    if dictionary.count /= 1 or test_error_messages then
      print_error_1(error_num+7, "put defensive count not one on one entry, actual is"
                 , dictionary.count)
    end

  -- Check one valid entry inserted correctly
    if (dictionary.at("1")) /= 1 or test_error_messages then
      print_error_1(error_num+8, "put defensive first entry value not 1, actual at is"
                 , dictionary.at("1"))
    end
    if dictionary @ "1" /= 1 or test_error_messages then
      print_error_1(error_num+9, "put defensive first entry value not 1, actual infix at is"
                 , dictionary @ "1")
    end

  -- Check count is decremented on remove first entry
    dictionary.remove("1")
    if dictionary.count /= 0 or test_error_messages then
      print_error_1(error_num+10, "put defensive first entry remove count is not 0, actual is"
                 , dictionary.count)
    end

  -- Check one valid entry removed correctly
    if dictionary.has("1") or test_error_messages then
      print_err_msg(error_num+11, "put defensive first entry value remove still has string 1")
    end
  end

---------------------------------------------------------
feature -- Many entry tests

multiple_entry_cases(size : INTEGER; count : INTEGER error_num : INTEGER)
  -- test cases with many entries in a dictionary
  local
  	dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
 -- Insert with put cases
  	create dictionary.make(size)
    insert_put(dictionary,count)

  -- Check has with many entries
    from k := 1
    until k > count
    loop
      if not dictionary.has(k) or test_error_messages then
        print_error_1(error_num, "Put does not have value", k)
      end
      k := k + 1
    end

  -- Check count is correct
    if dictionary.count /= count or test_error_messages then
      print_error_2(error_num+1, "put count is not", count, "actual is"
                 , dictionary.count)
    end

  -- Check entries have correct values
    from k := 1
    until k > count
    loop
      if dictionary.at(k) /= k*2 or test_error_messages then
      	print_error_2(error_num+2, "put at value is not", k*2, "actual is"
                 , dictionary.at(k))
      end
      if dictionary @ k /= k*2 or test_error_messages then
       print_error_2(error_num+3, "put infix at is not", k*2, "actual is"
                 , dictionary @ k)
      end
      k := k + 1
    end

  -- Check count is decremented
    dictionary.remove(2)
    if dictionary.count /= count - 1 or test_error_messages then
      print_error_2(error_num+4, "put remove count is not", count-1, "actual is"
                 , dictionary.count)
    end

  -- Check does not have removed entry
    if dictionary.has(2) or test_error_messages then
      print_error_1(error_num+5, "Put remove till has after remove of", 2)
    end

  --------------------------
  -- Test with put_defensive
  	create dictionary.make(size)
    insert_put_defensive(dictionary,count)

  -- Check has with many entries
    from k := 1
    until k > count
    loop
      if not dictionary.has(k) or test_error_messages then
        print_error_1(error_num+6, "Put defensive does not have value", k)
      end
      k := k + 1
    end

  -- Check count is correct
    if dictionary.count /= count or test_error_messages then
      print_error_2(error_num+7, "put defensive count is not", count, "actual is"
                 , dictionary.count)
    end

  -- Check entries have correct values
    from k := 1
    until k > count
    loop
      if dictionary.at(k) /= k*2 or test_error_messages then
      	print_error_2(error_num+8, "put defensive at value is not", k*2, "actual is"
                 , dictionary.at(k))
      end
      if dictionary @ k /= k*2 or test_error_messages then
       print_error_2(error_num+9, "put defensive infix at is not", k*2, "actual is"
                 , dictionary @ k)
      end
      k := k + 1
     end

  -- Check count is decremented
    dictionary.remove(2)
    if dictionary.count /= count - 1 or test_error_messages then
      print_error_2(error_num+10, "Put defensive remove count is not", count-1, "actual is"
                 , dictionary.count)
    end

  -- Check does not have removed entry
    if dictionary.has(2) or test_error_messages then
      print_error_1(error_num+11, "Put defensive remove still has after remove of", 2)
    end
end

------------------------------------------------------------------------------
feature { NONE } -- Test support

print_err_msg(i : INTEGER; s : STRING)
  do
    io.put_integer(i) ;
    io.put_character(' ') ; io.put_string(s) ; io.put_character(' ')
    io.new_line
  end

println(s : STRING)
  do
    io.put_string(s) ; io.new_line
  end

print3(s1 : STRING; row : INTEGER; s2 : STRING; column : INTEGER; s3 : STRING)
  do
    io.put_string(s1) ; io.put_integer(row) ; io.put_string(s2)
    io.put_integer(column) ; io.put_string(s3) ; io.new_line
  end

print_error_1(error_number : INTEGER; msg : STRING; actual_value : INTEGER)
  -- Print error message with expected value (in message) and actual value.
  do
    io.put_INTEGER(error_number)
    io.put_character(' ') ;io.put_string(msg) ; io.put_character(' ');
    io.put_integer(actual_value)
    io.new_line
  end

print_error_2( error_number : INTEGER ; msg1 : STRING
             ; expected_value : INTEGER
             ; msg2 : STRING ;actual_value : INTEGER)
  -- Print error message with expected value and actual value.
  do
    io.put_INTEGER(error_number)
    io.put_character(' ') ;io.put_string(msg1); io.put_character(' ');
    io.put_integer(expected_value)
    io.put_character(' ') ;io.put_string(msg2); io.put_character(' ');
    io.put_integer(actual_value)
    io.new_line
  end

end
