--| TEST_DICT
--|-----------------------------------------------------------------------------
--|   Copyright (C) GunnarGotshalks --  All rights reserved.
--|
--| Permission to use, copy, modify, and distribute,without fee this
--| software and its documentation for educational purposes and is
--| hereby granted.
--|-----------------------------------------------------------------------------

note
  description: "Use ES_TEST to test DICTIONARY class"
  author: "Gunnar Gotshalks"
  date: "2003 July 16"
  version: "2.0"

class TEST_DICT
  inherit ES_TEST

create make

feature -- Creation

do_precondition_cases : BOOLEAN = true

make
  -- Defines the test cases to use
  do
    add_boolean_case (agent test_boolean)
    add_violation_case(agent test_precondition)
    creation_cases
    void_precond_cases
    has_precond_cases
    space_precond_cases
    one_entry_cases
    multiple_entry_cases
    last_entry_cases
  end

test_boolean : BOOLEAN
  do
    comment("test boolean")
    Result := true
  end

test_precondition
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
    comment("test violation")
    create dictionary.make(5)
  end

feature -- Create tests

creation_cases
  do
-- Test creation of an empty dictionary.
  	add_boolean_case(agent create_creates(10))
  	add_boolean_case(agent create_capacity_set(10))
  	add_boolean_case(agent create_zero_count(10))
  	add_boolean_case(agent create_is_empty(10))
  	add_boolean_case(agent create_not_full(10))
  	add_boolean_case(agent create_creates(11))
  	add_boolean_case(agent create_capacity_set(11))
  	add_boolean_case(agent create_zero_count(11))
  	add_boolean_case(agent create_is_empty(11))
  	add_boolean_case(agent create_not_full(11))
end

create_too_small(size : INTEGER)
  -- Does precondition too small fail?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Precondition creation size too small")
  	create dictionary.make(size)
  end

create_creates(size : INTEGER) : BOOLEAN
  -- Does DICTIONARY create an object?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Is created dictionary non-void")
  	create dictionary.make(size)
  	Result := dictionary /= void
  end

create_capacity_set(size : INTEGER) : BOOLEAN
  -- Is the correct capacity dictionary created?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Is the capacity correct?")
  	create dictionary.make(size)
  	Result := dictionary.capacity = size
  end

create_zero_count(size : INTEGER) : BOOLEAN
  -- Is the correct size dictionary created?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Is a created dictionary size 0?")
  	create dictionary.make(size)
  	Result := dictionary.count = 0
  end

create_is_empty(size : INTEGER) : BOOLEAN
  -- Is the dictionary empty?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Is a created dictionary empty?")
  	create dictionary.make(size)
  	Result := dictionary.is_empty
  end

create_not_full(size : INTEGER) : BOOLEAN
  -- Is the dictionary full?
  local dictionary : DICTIONARY[INTEGER, STRING]
  do
  	comment("Is a created dictionary not full?")
  	create dictionary.make(size)
  	Result := not dictionary.is_full
  end

--------------------------------------
feature -- Pre key void tests

void_precond_cases
  do
-- Test void key trapped as precondition sin every relevent routine
  if do_precondition_cases then
  	add_violation_case(agent create_too_small(9))
  	add_violation_case(agent has_void_key(10))
  	add_violation_case(agent at_void_key(10))
  	add_violation_case(agent infix_at_void_key(10))
  	add_violation_case(agent put_void_key(10))
  	add_violation_case(agent put_defensive_void_key(10))
  	add_violation_case(agent replace_void_key(10))
  	add_violation_case(agent remove_void_key(10))
  end
end

has_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : BOOLEAN
  do
  	comment("Precondition has key not void")
  	create dictionary.make(size)
  	temp_result := dictionary.has(void)
  end

at_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
  	comment("Precondition at key /= void")
    create dictionary.make(size)
    temp_result := dictionary.at(void)
  end

infix_at_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
  	comment("Precondition @ key /= void")
    create dictionary.make(size)
    temp_result := dictionary @ void
  end

put_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition put key /= void")
    create dictionary.make(size)
    dictionary.put(void, "one")
  end

put_defensive_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition put_defensive key /= void")
    create dictionary.make(size)
    dictionary.put_defensive(void, "one")
  end

replace_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition replace key /= void")
    create dictionary.make(size)
    dictionary.replace(void, "one")
  end

remove_void_key(size : INTEGER)
  -- Check for precondition key /= void
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition remove key /= void")
    create dictionary.make(size)
   dictionary.remove(void)
  end

--------------------------------------
feature -- Pre has tests

has_precond_cases
-- Test has trapped as precondition in every relevent routine
  do if do_precondition_cases then
  	add_violation_case(agent at_not_has(10))
  	add_violation_case(agent infix_at_not_has(10))
  	add_violation_case(agent put_has(10))
  	add_violation_case(agent remove_not_has(10))
  end
end

at_not_has(size : INTEGER)
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
  	comment("Precondition at has key")
    create dictionary.make(size)
    temp_result := dictionary.at("1")
  end

infix_at_not_has(size : INTEGER)
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  temp_result : STRING
  do
  	comment("Precondition @ has key")
    create dictionary.make(size)
    temp_result := dictionary @ "1"
  end

put_has(size : INTEGER)
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition put has key")
    create dictionary.make(size)
    dictionary.put("1","one")
    dictionary.put("1","two")
  end

remove_not_has(size : INTEGER)
  -- Check precondition has key
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Precondition remove does not have key")
    create dictionary.make(size)
    dictionary.remove("1")
  end

--------------------------------------
feature -- Pre space exists tests

space_precond_cases
-- Test space_exists trapped as precondition in every relevent routine
  do if do_precondition_cases then
  	add_violation_case(agent put_no_space(10))
  	add_violation_case(agent put_defensive_no_space(10))
  end
end

put_no_space(size : INTEGER)
  -- Check space exists precondition.
  local
    dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
  	comment("Precondition put space does not exist")
  	create dictionary.make(size)
    insert_put(dictionary, size)
    dictionary.put(k,1001)
  end

put_defensive_no_space(size : INTEGER)
  -- Check space exists precondition.
  local
    dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
  	comment("Precondition put_defensive space does not exist")
  	create dictionary.make(size)
    insert_put_defensive(dictionary, size)
    dictionary.put_defensive(k,1001)
  end

insert_put_defensive( dictionary : DICTIONARY[INTEGER,INTEGER]
                    ; xcount : INTEGER)
  local k : INTEGER
  do
    from k := 1
  	invariant -- for j : 1..k-1 :: an entry is in the dictionary
  	until k > xcount
  	loop
  	  dictionary.put_defensive(k,k*2)
  	  k := k + 1
  	variant xcount + 1 - k
  	end
  end

insert_put( dictionary : DICTIONARY[INTEGER,INTEGER]
          ; xcount : INTEGER)
  local k : INTEGER
  do
    from k := 1
  	invariant -- for j : 1..k-1 :: an entry is in the dictionary
  	until k > xcount
  	loop
  	  dictionary.put(k,k*2)
  	  k := k + 1
  	variant xcount + 1 - k
  	end
  end

--------------------------------------
feature -- One entry tests

one_entry_cases
-- Test inserting one entry and removing one entry
  do
   	add_boolean_case(agent put_one_has(10,1))
  	add_boolean_case(agent put_one_has(10,2))
  	add_boolean_case(agent put_one_count(10,1))
  	add_boolean_case(agent put_one_count(10,2))
  	add_boolean_case(agent put_one_value(10,1))
  	add_boolean_case(agent put_one_value(10,2))
  	add_boolean_case(agent remove_one_value(10))
  	add_boolean_case(agent remove_one_count(10))
end

put_one_has(size : INTEGER; which : INTEGER) : BOOLEAN
  -- Check one valid entry inserted correctly
  -- which = 1 use put -- which = 2 use put_defensive
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Put one entry key is correct")
  	create dictionary.make(size)
    if which = 1 then dictionary.put("1", "one")
    else dictionary.put_defensive("1","one")
    end
    Result := dictionary.has("1")
  end

put_one_count(size : INTEGER; which : INTEGER) : BOOLEAN
  -- Check one entry with count of 1
  -- which = 1 use put -- which = 2 use put_defensive
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Put one entry count is correct")
  	create dictionary.make(size)
    if which = 1 then dictionary.put("1", "one")
    else dictionary.put_defensive("1","one")
    end
    Result := dictionary.count = 1
  end

put_one_value(size : INTEGER; which : INTEGER) : BOOLEAN
  -- Check one valid entry inserted correctly
  -- which = 1 use put -- which = 2 use put_defensive
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("Put one entry value is correct")
  	create dictionary.make(size)
    if which = 1 then dictionary.put("1", "one")
    else dictionary.put_defensive("1","one")
    end
    Result := (dictionary.at("1")).is_equal("one")
              and (dictionary @ "1").is_equal("one")
  end

remove_one_count(size : INTEGER) : BOOLEAN
  -- Check count is decremented
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("remove one entry count is decremented")
  	create dictionary.make(size)
    dictionary.put("1", "one")
    dictionary.remove("1")
    Result := dictionary.count = 0
  end

remove_one_value(size : INTEGER) : BOOLEAN
  -- Check one valid entry removed correctly
  local dictionary : DICTIONARY[STRING, STRING]
  do
  	comment("remove one entry key not there")
  	create dictionary.make(size)
    dictionary.put("1", "one")
    dictionary.remove("1")
    Result := not dictionary.has("1")
  end

---------------------------------------------------------
feature -- Many entry tests

multiple_entry_cases
-- Test inserting multiple enties and removing mulltiple entries
  do
    add_boolean_case(agent put_many_has(10,5,1))
  	add_boolean_case(agent put_many_has(10,5,2))
  	add_boolean_case(agent put_many_count(10,5,1))
  	add_boolean_case(agent put_many_count(10,5,2))
  	add_boolean_case(agent put_many_value(10,5,1))
  	add_boolean_case(agent put_many_value(10,5,2))
  	add_boolean_case(agent remove_many_value(10,5))
  	add_boolean_case(agent remove_many_count(10,5))
end

last_entry_cases
-- Test inserting last entry and removing last entry
  do
   	add_boolean_case(agent put_many_has(10,10,1))
  	add_boolean_case(agent put_many_has(10,10,2))
  	add_boolean_case(agent put_many_count(10,10,1))
  	add_boolean_case(agent put_many_count(10,10,2))
  	add_boolean_case(agent put_many_value(10,10,1))
  	add_boolean_case(agent put_many_value(10,10,2))
  	add_boolean_case(agent remove_many_value(10,10))
  	add_boolean_case(agent remove_many_count(10,10))
end

put_many_has( size : INTEGER; xcount : INTEGER
            ; which : INTEGER) : BOOLEAN
  -- Check many entries inserted correctly
  -- which = 1 use put -- which = 2 use put_defensive
  local
  	dictionary : DICTIONARY[INTEGER, INTEGER]
    k : INTEGER
  do
  	comment("Put many has correct keys")
  	create dictionary.make(size)
    if which = 1 then
    	comment("Put many has correct keys")
        insert_put(dictionary,xcount)
    else
    	comment("Put_defensive many has correct keys")
	  	insert_put_defensive(dictionary,xcount)
    end
    Result := true
    from k := 1
    until k > xcount
    loop
      Result := dictionary.has(k) and Result
      k := k + 1
    end
  end

put_many_count( size : INTEGER; xcount : INTEGER
            ;  which : INTEGER) : BOOLEAN
  -- Check one valid entry inserted correctly
  -- which = 1 use put -- which = 2 use put_defensive
  local dictionary : DICTIONARY[INTEGER, INTEGER]
  do
  	create dictionary.make(size)
    if which = 1 then
    	comment("Put many has correct count")
        insert_put(dictionary,xcount)
    else
    	comment("Put_defensive many has correct count")
	  	insert_put_defensive(dictionary,xcount)
    end
    Result := dictionary.count = xcount
  end

put_many_value( size : INTEGER; xcount : INTEGER
            ;  which : INTEGER) : BOOLEAN
  -- Check one valid entry inserted correctly
  -- which = 1 use put -- which = 2 use put_defensive
  local
  	dictionary : DICTIONARY[INTEGER, INTEGER]
  	k : INTEGER
  do
  	comment("Put many has correct values")
  	create dictionary.make(size)
    if which = 1 then
    	comment("Put many has correct values")
        insert_put(dictionary,xcount)
    else
    	comment("Put_defensive many has correct values")
	  	insert_put_defensive(dictionary,xcount)
    end
    Result := true
    from k := 1
    until k > xcount
    loop
      if which = 1 then
        Result := dictionary.at(k) = k*2 and Result
      else
        Result := dictionary @ k = k*2 and Result
      end
    k := k + 1
    end
  end

remove_many_count(size : INTEGER; xcount : INTEGER) : BOOLEAN
  -- Check count is decremented
  local dictionary : DICTIONARY[INTEGER, INTEGER]
  do
    comment("Remove one from many has correct count")
  	create dictionary.make(size)
    insert_put(dictionary,xcount)
    dictionary.remove(2)
    Result := dictionary.count = xcount -1
  end

remove_many_value(size : INTEGER; xcount : INTEGER) : BOOLEAN
  -- Check one valid entry removed correctly
  local dictionary : DICTIONARY[INTEGER, INTEGER]
  do
  	comment("Remove one from many does not have key")
  	create dictionary.make(size)
    insert_put(dictionary,xcount)
    dictionary.remove(2)
    Result := not dictionary.has(2)
  end

end
